@@ -18,10 +18,10 @@
  */
 typedef struct {
     PyObject_HEAD
+    builder_t builder;
     PyObject *target_send;
-    builder_t *builder;
     PyObject *prefix;
-    PyObject *end_event;
+    int object_depth;
 } ItemsBasecoro;
 
 /**
