@@ -2,6 +2,9 @@
 Backend independent higher level interfaces, common exceptions.
 '''
 import decimal
+import inspect
+import warnings
+
 from ijson import compat, utils
 
 
@@ -143,11 +146,15 @@ def items_basecoro(target, prefix, map_type=None):
         current, event, value = (yield)
         if current == prefix:
             if event in ('start_map', 'start_array'):
+                object_depth = 1
                 builder = ObjectBuilder(map_type=map_type)
-                end_event = event.replace('start', 'end')
-                while (current, event) != (prefix, end_event):
+                while object_depth:
                     builder.event(event, value)
                     current, event, value = (yield)
+                    if event in ('start_map', 'start_array'):
+                        object_depth += 1
+                    elif event in ('end_map', 'end_array'):
+                        object_depth -= 1
                 del builder.containers[:]
                 target.send(builder.value)
             else:
@@ -164,17 +171,26 @@ def kvitems_basecoro(target, prefix, map_type=None):
     while True:
         path, event, value = (yield)
         while path == prefix and event == 'map_key':
+            object_depth = 0
             key = value
             builder = ObjectBuilder(map_type=map_type)
             path, event, value = (yield)
-            while path != prefix:
+            if event == 'start_map':
+                object_depth += 1
+            while (
+                (event != 'map_key' or object_depth != 0) and
+                (event != 'end_map' or object_depth != -1)):
                 builder.event(event, value)
                 path, event, value = (yield)
+                if event == 'start_map':
+                    object_depth += 1
+                elif event == 'end_map':
+                    object_depth -= 1
             del builder.containers[:]
             target.send((key, builder.value))
 
 
-def number(str_value):
+def integer_or_decimal(str_value):
     '''
     Converts string with a numeric value into an int or a Decimal.
     Used in different backends for consistent number representation.
@@ -183,13 +199,22 @@ def number(str_value):
         return int(str_value)
     return decimal.Decimal(str_value)
 
+def integer_or_float(str_value):
+    '''
+    Converts string with a numeric value into an int or a float.
+    Used in different backends for consistent number representation.
+    '''
+    if not ('.' in str_value or 'e' in str_value or 'E' in str_value):
+        return int(str_value)
+    return float(str_value)
 
-def file_source(f, use_string_reader, buf_size=64*1024):
+def number(str_value):
+    warnings.warn("number() function will be removed in a later release", DeprecationWarning)
+    return integer_or_decimal(str_value)
+
+def file_source(f, buf_size=64*1024):
     '''A generator that yields data from a file-like object'''
-    if use_string_reader:
-        f = compat.string_reader(f)
-    else:
-        f = compat.bytes_reader(f)
+    f = compat.bytes_reader(f)
     while True:
         data = f.read(buf_size)
         yield data
@@ -254,53 +279,165 @@ def _make_items_coro(backend):
 
 
 def _make_kvitems_coro(backend):
-    def kvitems(target, prefix, map_type=None, **config):
+    def kvitems_coro(target, prefix, map_type=None, **config):
         return utils.chain(
             target,
             *_kvitems_pipeline(backend, prefix, map_type, config)
         )
-    return kvitems
+    return kvitems_coro
 
 
-def _make_basic_parse(backend, use_string_reader):
-    def basic_parse(f, buf_size=64*1024, **config):
+def is_awaitablefunction(func):
+    """True if `func` is an awaitable function"""
+    return (
+        inspect.iscoroutinefunction(func) or (
+          inspect.isgeneratorfunction(func) and
+          (func.__code__.co_flags & inspect.CO_ITERABLE_COROUTINE)
+        )
+    )
+
+def is_async_file(f):
+    """True if `f` has an asynchronous `read` method"""
+    return (
+        compat.IS_PY35 and hasattr(f, 'read') and
+        is_awaitablefunction(f.read)
+    )
+
+def is_file(x):
+    """True if x has a `read` method"""
+    return hasattr(x, 'read')
+
+
+def is_iterable(x):
+    """True if x can be iterated over"""
+    return hasattr(x, '__iter__')
+
+
+def _get_source(source):
+    if isinstance(source, compat.bytetype):
+        return compat.BytesIO(source)
+    elif isinstance(source, compat.texttype):
+        return compat.StringIO(source)
+    return source
+
+
+def _make_basic_parse_gen(backend):
+    def basic_parse_gen(file_obj, buf_size=64*1024, **config):
         return utils.coros2gen(
-            file_source(f, use_string_reader, buf_size=buf_size),
+            file_source(file_obj, buf_size=buf_size),
             *_basic_parse_pipeline(backend, config)
         )
+    return basic_parse_gen
+
+
+def _make_parse_gen(backend):
+    def parse_gen(file_obj, buf_size=64*1024, **config):
+        return utils.coros2gen(
+            file_source(file_obj, buf_size=buf_size),
+            *_parse_pipeline(backend, config)
+        )
+    return parse_gen
+
+
+def _make_items_gen(backend):
+    def items_gen(file_obj, prefix, map_type=None, buf_size=64*1024, **config):
+        return utils.coros2gen(
+            file_source(file_obj, buf_size=buf_size),
+            *_items_pipeline(backend, prefix, map_type, config)
+        )
+    return items_gen
+
+
+def _make_kvitems_gen(backend):
+    def kvitems_gen(file_obj, prefix, map_type=None, buf_size=64*1024, **config):
+        return utils.coros2gen(
+            file_source(file_obj, buf_size=buf_size),
+            *_kvitems_pipeline(backend, prefix, map_type, config)
+        )
+    return kvitems_gen
+
+
+def _make_basic_parse(backend):
+    def basic_parse(source, buf_size=64*1024, **config):
+        source = _get_source(source)
+        if is_async_file(source):
+            return backend['basic_parse_async'](
+                source, buf_size=buf_size, **config
+            )
+        elif is_file(source):
+            return backend['basic_parse_gen'](
+                source, buf_size=buf_size, **config
+            )
+        raise ValueError("Unknown source type: %r" % type(source))
     return basic_parse
 
 
-def _make_parse(backend, use_string_reader):
-    def parse(f, buf_size=64*1024, **config):
-        return utils.coros2gen(
-            file_source(f, use_string_reader, buf_size=buf_size),
-            *_parse_pipeline(backend, config)
-        )
+def _make_parse(backend):
+    def parse(source, buf_size=64*1024, **config):
+        source = _get_source(source)
+        if is_async_file(source):
+            return backend['parse_async'](
+                source, buf_size=buf_size, **config
+            )
+        elif is_file(source):
+            return backend['parse_gen'](
+                source, buf_size=buf_size, **config
+            )
+        elif is_iterable(source):
+            return utils.coros2gen(source,
+                (parse_basecoro, (), {})
+            )
+        raise ValueError("Unknown source type: %r" % type(source))
     return parse
 
 
-def _make_items(backend, use_string_reader):
-    def items(f, prefix, map_type=None, buf_size=64*1024, **config):
-        return utils.coros2gen(
-            file_source(f, use_string_reader, buf_size=buf_size),
-            *_items_pipeline(backend, prefix, map_type, config)
-        )
+def _make_items(backend):
+    def items(source, prefix, map_type=None, buf_size=64*1024, **config):
+        source = _get_source(source)
+        if is_async_file(source):
+            return backend['items_async'](
+                source, prefix, map_type=map_type, buf_size=buf_size, **config
+            )
+        elif is_file(source):
+            return backend['items_gen'](
+                source, prefix, map_type=map_type, buf_size=buf_size, **config
+            )
+        elif is_iterable(source):
+            return utils.coros2gen(source,
+                (backend['items_basecoro'], (prefix,), {'map_type': map_type})
+            )
+        raise ValueError("Unknown source type: %r" % type(source))
     return items
 
 
-def _make_kvitems(backend, use_string_reader):
-    def kvitems(f, prefix, map_type=None, buf_size=64*1024, **config):
-        return utils.coros2gen(
-            file_source(f, use_string_reader, buf_size=buf_size),
-            *_kvitems_pipeline(backend, prefix, map_type, config)
-        )
+def _make_kvitems(backend):
+    def kvitems(source, prefix, map_type=None, buf_size=64*1024, **config):
+        source = _get_source(source)
+        if is_async_file(source):
+            return backend['kvitems_async'](
+                source, prefix, map_type=map_type, buf_size=buf_size, **config
+            )
+        elif is_file(source):
+            return backend['kvitems_gen'](
+                source, prefix, map_type=map_type, buf_size=buf_size, **config
+            )
+        elif is_iterable(source):
+            return utils.coros2gen(source,
+                (backend['kvitems_basecoro'], (prefix,), {'map_type': map_type})
+            )
+        raise ValueError("Unknown source type: %r" % type(source))
     return kvitems
 
 
+_common_functions_warn = '''
+Don't use the ijson.common.* functions; instead go directly with the ijson.* ones.
+See the documentation for more information.
+'''
+
 def parse(events):
     """Like ijson.parse, but takes events generated via ijson.basic_parse instead
     of a file"""
+    warnings.warn(_common_functions_warn, DeprecationWarning)
     return utils.coros2gen(events,
         (parse_basecoro, (), {})
     )
@@ -309,6 +446,7 @@ def parse(events):
 def kvitems(events, prefix, map_type=None):
     """Like ijson.kvitems, but takes events generated via ijson.parse instead of
     a file"""
+    warnings.warn(_common_functions_warn, DeprecationWarning)
     return utils.coros2gen(events,
         (kvitems_basecoro, (prefix,), {'map_type': map_type})
     )
@@ -317,31 +455,35 @@ def kvitems(events, prefix, map_type=None):
 def items(events, prefix, map_type=None):
     """Like ijson.items, but takes events generated via ijson.parse instead of
     a file"""
+    warnings.warn(_common_functions_warn, DeprecationWarning)
     return utils.coros2gen(events,
         (items_basecoro, (prefix,), {'map_type': map_type})
     )
 
 
-def enrich_backend(backend, use_string_reader=False):
+def enrich_backend(backend):
     '''
     Provides a backend with any missing coroutines/generators/async-iterables
     it might be missing by using the generic ones written in python.
     '''
     backend['backend'] = backend['__name__'].split('.')[-1]
-    for gen_name in ('basic_parse', 'parse', 'items', 'kvitems'):
-        basecoro_name = gen_name + '_basecoro'
+    for name in ('basic_parse', 'parse', 'items', 'kvitems'):
+        basecoro_name = name + '_basecoro'
         if basecoro_name not in backend:
             backend[basecoro_name] = globals()[basecoro_name]
-        coro_name = gen_name + '_coro'
+        coro_name = name + '_coro'
         if coro_name not in backend:
             factory = globals()['_make_' + coro_name]
             backend[coro_name] = factory(backend)
+        gen_name = name + '_gen'
         if gen_name not in backend:
             factory = globals()['_make_' + gen_name]
-            backend[gen_name] = factory(backend, use_string_reader)
+            backend[gen_name] = factory(backend)
         if compat.IS_PY35:
             from . import utils35
-            async_name = gen_name + '_async'
+            async_name = name + '_async'
             if async_name not in backend:
                 factory = getattr(utils35, '_make_' + async_name)
-                backend[async_name] = factory(backend, use_string_reader)
\ No newline at end of file
+                backend[async_name] = factory(backend)
+        factory = globals()['_make_' + name]
+        backend[name] = factory(backend)
\ No newline at end of file
