@@ -1,3 +1,4 @@
+import importlib.util
 import unittest
 
 from ijson import common, compat
@@ -16,6 +17,15 @@ class Misc(unittest.TestCase):
         self.assertEqual(len(warns), 1)
         self.assertEqual(DeprecationWarning, warns[0].category)
 
+    def test_yajl2_c_loadable(self):
+        if compat.IS_PY2:
+            self.skipTest("Test requires Python 3.4+")
+        spec = importlib.util.find_spec("ijson.backends._yajl2")
+        if spec is None:
+            self.skipTest("yajl2_c is not built")
+        importlib.util.module_from_spec(spec)
+
+
 class MainEntryPoints(object):
 
     def _assert_invalid_type(self, routine, *args, **kwargs):
